
procedure FFTIFT(InputReal,InputImag: WaveformPointer; Direction: Char);

{Forward or Inverse Fourier Transform using power-of-two algorithm based
 on Sande-Tukey (decimation in frequency) algorithm as outlined in The
 FFT: Fundamentals and Concepts by Robert W. Ramirez (Tektronix, Inc.)}

{Input parameters point to 2 arrays, [0..WaveformSizeM1], of data to be
 transformed (WaveformSize = 2^WaveformPower); Direction = 'F' or 'I'}

{Assumes existence of following constants:
    WaveformPower      (e.g., 9)
    WaveformSize       (e.g., 512)
    WaveformSizeM1     (e.g., 511)
    HalfWaveformSize   (e.g., 256)
    HalfWaveformSizeM1 (e.g., 255)
  & Type of WaveformPointer}

type
  HalfWaveformArray = Array[0..HalfWaveformSizeM1] of Real;
  HalfWaveformPointer = ^HalfWaveFormArray;

var
  N,M,Sign,I,L,H,G,K,TFI,TFIFlag,J,R,S,T,Index,Iout,Temp : Integer;
  Pi2,TempReal,TempImag : Real;
  BufferReal,BufferImag : WaveformPointer;
  TFReal,TFImag : HalfWaveformPointer;
  TempPointer : ^Integer;

function IntPower(X,Y: Integer) : Integer;
var I,J : Integer;
begin {of function IntPower}
J := 1;
for I := 1 to Y do
  J := J * X;
IntPower := J;
end; {of function IntPower}

begin {of procedure FFTIFT}
N := WaveformSize;
M := WaveformPower;
Mark(TempPointer);                     {Mark current heap size}
New(BufferReal); New(BufferImag);      {Allocate memory for Buffers}
New (TFReal); New(TFImag);             {Allocate memory for Twiddle Factors}
if ((Direction = 'F') or (Direction = 'f'))   {Establish Transform Direction}
then Sign := -1
else Sign := 1;
if Sign = 1                            {Reorder data for Inverse FT}
then
  begin
  for I := 0 to (N div 2) do
    begin
    BufferReal^[I] := InputReal^[I+((N div 2)-1)];
    BufferImag^[I] := InputImag^[I+((N div 2)-1)];
    end;
  for I := 0 to ((N div 2)-2) do
    begin
    BufferReal^[I+((N div 2)+1)] := InputReal^[I];
    BufferImag^[I+((N div 2)+1)] := InputImag^[I];
    end;
  for I := 0 to (N-1) do
    begin
    InputReal^[I] := BufferReal^[I];
    InputImag^[I] := BufferImag^[I];
    end;
  end;
Pi2 := 2.0 * Pi;                       {Generate Twiddle Factors}
for I := 0 to ((N div 2)-1) do
  begin
  TFReal^[I] := Cos(Pi2*(-I)/N);
  TFImag^[I] := Sign*Sin(Pi2*(-I)/N);
  end;
for I := 1 to M do                     {Compute Fast Fourier Transform}
  begin
  L := 0; H := 0;
  G := N div IntPower(2,I);
  K := 0;
  while K <= (N-1) do {step G}
    begin
    TFI := 0;
    TFIFlag := IntPower(-1,L+1);
    for J := 0 to (G-1) do
      begin
      TFI := J*IntPower(2,I-1);
      R := K + J;
      S := J + H;
      T := J + G + H;
      if TFIFlag < 0
      then
        begin
        BufferReal^[R] := InputReal^[S] + InputReal^[T];
        BufferImag^[R] := InputImag^[S] + InputImag^[T]
        end
      else
        begin
        TempReal := InputReal^[S] - InputReal^[T];
        TempImag := InputImag^[S] - InputImag^[T];
        BufferReal^[R] := TempReal*TFReal^[TFI] - TempImag*TFImag^[TFI];
        BufferImag^[R] := TempReal*TFImag^[TFI] + TempImag*TFReal^[TFI];
        end;
      end;
    L := L + 1; H := Trunc(L/2)*G*2;
    K := K + G;
    end;
  for J := 0 to (N-1) do
    begin
    InputReal^[J] := BufferReal^[J];
    InputImag^[J] := BufferImag^[J];
    end;
  end;
for I := 0 to (N-1) do
  begin
  InputReal^[I] := InputReal^[I]/N;
  InputImag^[I] := InputImag^[I]/N;
  end;
for I := 0 to (N-1) do                 {Bit reversal routine to}
  begin                                {unscramble FFT results}
  Index := I;
  Iout := 0;
  for J := 1 to M do
    begin
    Temp := 1 and Index;
    Iout := Iout * 2;
    Iout := Iout + Temp;
    Index := Index div 2;
    end;
  BufferReal^[I] := InputReal^[Iout];
  BufferImag^[I] := InputImag^[Iout];
  end;
if Sign = -1
then
  begin                                {Order FFT output for neg freq at 0}
  for I := 0 to (N div 2) do           {to N/2-2, DC at N/2-1, pos freq at}
    begin                              {N/2 to N-2, and Nyquist at N-1}
    InputReal^[I+((N div 2)-1)] := BufferReal^[I];
    InputImag^[I+((N div 2)-1)] := BufferImag^[I];
    end;
  for I := 0 to ((N div 2)-2) do
    begin
    InputReal^[I] := BufferReal^[I+((N div 2)+1)];
    InputImag^[I] := BufferImag^[I+((N div 2)+1)]
    end
  end
else
  begin                                {Scale IFT output by factor of N}
  for I := 0 to (N-1) do
    begin
    InputReal^[I] := BufferReal^[I]*N;
    InputImag^[I] := BufferImag^[I]*N;
    end;
  end;
Release(TempPointer);                  {Deallocate temporary arrays in heap}
end; {of procedure FFTIFT}

procedure FFT(InputReal,InputImag: WaveformPointer);
begin {of procedure FFT}
FFTIFT(InputReal,InputImag,'F');
end; {of procedure FFT}

procedure IFT(InputReal,InputImag: WaveformPointer);
begin {of procedure IFT}
FFTIFT(InputReal,InputImag,'I');
end; {of procedure IFT}

procedure Polar(InputReal,InputImag: WaveformPointer);
const
  Threshold = 1E-6;
var
  i: Integer;
  Magnitude,Phase,
  MaxMag,ThreshMag: Real;
begin {of procedure Polar}
MaxMag := 0;
for i := 0 to WaveformSizeM1 do
  begin
  Magnitude := Abs(Sqrt(Sqr(InputReal^[i])+Sqr(InputImag^[i])));
  if Magnitude > MaxMag then MaxMag := Magnitude;
  end;
ThreshMag := Threshold*MaxMag;
for i := 0 to WaveformSizeM1 do
  begin
  Magnitude := Sqrt(Sqr(InputReal^[i])+Sqr(InputImag^[i]));
  if ((Abs(Magnitude) > ThreshMag) and (InputReal^[i] <> 0))
  then Phase := ArcTan(InputImag^[i]/InputReal^[i])
  else Phase := 0;
  InputReal^[i] := Magnitude;
  InputImag^[i] := Phase;
  end;
end; {of procedure Polar}

