

procedure PolyFit(X,Y,W: DataPointer; Npts,Nparam: Integer; var A,SigmaA:
                  FitArray; var ChiSqR: Real; var Yfit: DataPointer);

{make a least squares fit to data with a polynomial of the form

     Y = A(1) + A(2)*X + A(3)*X + A(4)*X + ...

 by Will Polik, UC Berkeley, 5/5/86

 based on Bevington, Data Reduction and Error Analysis for the Physical
 Sciences, (McGraw-Hill, 1969), Chapter 8.

 Assumes:
   const
     MaxMatSize = ??;
   type
     DataArray = Array[1..???] of Real;
     DataPointer = ^DataArray;
     FitArray  = Array[1..MaxMatSize] of Real;
     InvMatrix = Array[1..MaxMatSize,1..MaxMatSize] of Real;
  $I IPOWER.FNC
  $I SYMATINV.PRO

 Parameters:
   X      - Pointer to array of independent variable data points
   Y      - Pointer to array of dependent variable data points
   W      - Pointer to array of relative weights of data points
   Npts   - Number of pairs of data points
   Nparam - Number of fitting parameters (= polynomial order + 1)
   A      - Array of polynomial fit coefficients
   SigmaA - Array of polynomial fit standard deviations
   ChiSqR - Reduced chi-square for fit (standard deviation)
   Yfit   - Pointer to array of calculated y values}

var
  i,j,k  : Integer;
  Alpha  : InvMatrix;
  Beta   : Array[1..MaxMatSize] of Real;
  Determ : Double;
  ChiSq  : Real;
  Wtot   : Real;

begin {of procedure Polyfit}
if Nparam > MaxMatSize
then
  begin
  WriteLn('ERROR: Nparam = ',Nparam,' > MaxMatSize = ',MaxMatSize,' in PolyFit');
  halt;
  end;
if Nparam >= Npts
then
  begin
  WriteLn('ERROR: Nparam = ',Nparam,' >= Npts = ',Npts,' in PolyFit');
  halt;
  end;
for i := 1 to Nparam do                {Zero Alpha, Beta, A, SigmaA}
  begin
  for j := 1 to Nparam do
    Alpha[i,j] := 0;
  Beta[i] := 0;
  A[i] := 0;
  SigmaA[i] := 0;
  end;
for i := 1 to Npts do                  {Accumulate Alpha and Beta Matrices}
  begin
  for j := 1 to Nparam do
    begin
    for k := 1 to j do                 {Alpha is symmetric; do lower triangle}
      Alpha[j,k] := Alpha[j,k] + W^[i]*IPower(X^[i],j+k-2);
    Beta[j] := Beta[j] + W^[i]*Y^[i]*IPower(X^[i],j-1);
    end;
  end;
for j := 1 to Nparam do                {Symmetrize Alpha}
  for k := 1 to j-1 do
    Alpha[k,j] := Alpha[j,k];
SymMatInv(Alpha,Nparam,Determ);        {Invert Alpha to get Error matrix}
for i := 1 to Nparam do                {Calculate polynomial fit coefficients}
  for j := 1 to Nparam do
    A[i] := A[i] + Beta[j]*Alpha[i,j];
for i := 1 to Npts do                  {Calculate fit}
  begin
  Yfit^[i] := 0;
  for j := 1 to Nparam do
    Yfit^[i] := Yfit^[i] + A[j]*IPower(X^[i],j-1);
  end;
ChiSq := 0;                            {Calculate chi-square, weight normalization}
Wtot := 0;
for i := 1 to Npts do
  begin
  ChiSq := ChiSq + W^[i]*Sqr(Y^[i]-Yfit^[i]);
  Wtot := Wtot + W^[i];
  end;
ChiSq := ChiSq*Npts/Wtot;
ChiSqR := ChiSq/(Npts-Nparam);         {Calculate reduced chi-square}
for i := 1 to Nparam do                {Calculate polynomial fit standard deviations}
  SigmaA[i] := Sqrt(ChiSqR*Alpha[i,i]*(Wtot/Npts));
end; {of procedure PolyFit}
